@@ -1,4 +1,4 @@
-# $Id: Signals.pm 1903 2006-03-20 04:44:08Z rcaputo $
+# $Id: Signals.pm 1954 2006-05-08 06:30:44Z teknikill $
 
 # The data necessary to manage signals, and the accessors to get at
 # that data in a sane fashion.
@@ -6,7 +6,7 @@
 package POE::Resources::Signals;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 1954 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -153,7 +153,7 @@ sub _data_sig_finalize {
   unless (RUNNING_IN_HELL) {
     local $!;
     until ((my $pid = waitpid( -1, 0 )) == -1) {
-      _warn( "Child process PID:$pid reaped: $!\n" );
+      _warn( "Child process PID:$pid reaped: $!\n" ) if ($pid);
       $finalized_ok = 0;
     }
   }
